@@ -26,8 +26,8 @@
  *   that compiling, linking, and/or using OpenSSL is allowed.
  *   
  *   
- *   $Rev: 125 $
- *   $Date: 2008-07-09 12:46:03 +0200 (Wed, 09 Jul 2008) $
+ *   $Rev: 149 $
+ *   $Date: 2009-05-28 14:04:54 +0200 (Thu, 28 May 2009) $
  *   
  *   
 */
@@ -55,13 +55,16 @@
 #include "arg.h"
 #include "misc.h"
 /*-------------------------------------------------------------------------*/
-struct timeval tm_start;
+struct timeval tm_main_start;
+struct timeval tm_main_total;
 fsm_t *fsm = 0;
 /*-------------------------------------------------------------------------*/
 void usage() {
-  printf("ophcrack 3.0.1 by Objectif Securite (http://www.objectif-securite.ch)\n\n");
+  printf("ophcrack 3.3.0 by Objectif Securite (http://www.objectif-securite.ch)\n\n");
   printf("Usage: ophcrack [OPTIONS]\n");
   printf("Cracks Windows passwords with Rainbow tables\n\n");
+  printf("  -a              disable audit mode (default)\n");
+  printf("  -A              enable audit mode\n");
   printf("  -b              disable bruteforce\n");
   printf("  -B              enable bruteforce (default)\n");
   printf("  -c config_file  specify the config file to use\n");
@@ -71,9 +74,12 @@ void usage() {
   printf("  -f file         load hashes from the specified file (pwdump or session)\n");
   printf("  -g              disable GUI\n");
   printf("  -h              display this information\n");
+  printf("  -i              hide usernames\n");
+  printf("  -I              show usernames (default)\n");
   printf("  -l file         log all output to the specified file\n");
   printf("  -n num          specify the number of threads to use\n");
   printf("  -o file         write cracking output to file in pwdump format\n");
+  printf("  -p num          preload (0 none, 1 index, 2 index+end, 3 all default)\n"); 
   printf("  -q              quiet mode\n");
   printf("  -r              launch the cracking when ophcrack starts (GUI only)\n");
   printf("  -s              disable session auto-saving\n");
@@ -83,6 +89,7 @@ void usage() {
   printf("                  specify which table to use in the directory given by -d\n");
   printf("  -v              verbose\n");
   printf("  -w dir          load hashes from encrypted SAM file in directory dir\n");
+  printf("  -x              export data in CSV format to the file specified by -o\n");
   printf("\n\nExample:\tophcrack -g -d /path/to/tables -t xp_free_fast,0,3:vista_free -f in.txt\n\n");
   printf("\t\tLaunch ophcrack in command line using tables 0 and 3 in\n");
   printf("\t\t/path/to/tables/xp_free_fast and all tables in /path/to/tables/vista_free\n");
@@ -124,10 +131,10 @@ int load_tables(ophcrack_t *crack, arg_t *arg, list_t *table_str) {
   list_t *table_path = arg->table_path;
   list_t *tables = list_alloc();
 
-  for (tnd = table_str->head; tnd != 0; tnd = tnd->prev) {
+  for (tnd = table_str->head; tnd != 0; tnd = tnd->next) {
     char *tblstr = (char*)tnd->data;
     
-    for (dnd = table_path->head; dnd != 0; dnd = dnd->prev) {
+    for (dnd = table_path->head; dnd != 0; dnd = dnd->next) {
       char *dir = (char*)dnd->data;
       int ret = table_open(tables, dir, tblstr);
       
@@ -135,7 +142,7 @@ int load_tables(ophcrack_t *crack, arg_t *arg, list_t *table_str) {
       
       if (ret >= 0) {
 	if (!arg->use_gui && ret > 0) {
-	  if (!arg->quiet || arg->lfile != stdout)
+	  if (!arg->quiet || arg->lfile != stdout) {
 	    if (dir[0] != 0)
 	      fprintf(arg->lfile, 
 		      "Opened %d table(s) from %s/%s.\n", 
@@ -144,6 +151,7 @@ int load_tables(ophcrack_t *crack, arg_t *arg, list_t *table_str) {
 	      fprintf(arg->lfile,
 		      "Opened %d table(s) from %s.\n", 
 		      ret, tblstr);
+	  }
 	}
 
 	break;
@@ -180,13 +188,22 @@ int load_tables(ophcrack_t *crack, arg_t *arg, list_t *table_str) {
   
   list_nd_t *nd;
   
-  for (nd = tables->head; nd != 0; nd = nd->prev) {
+  for (nd = tables->head; nd != 0; nd = nd->next) {
     table_t *tbl = (table_t*)nd->data;
-    ophcrack_add_table(crack, tbl);
+
+    ophcrack_setup_table(tbl);
+
+    int ret = table_verify(tbl);
+    if (ret > 0)
+      ophcrack_add_table(crack, tbl);
+    else if (!arg->use_gui && ret < 0) 
+      fprintf(stderr, "Size of table(s) %s,%d is invalid. Download it again.\n", tbl->name, tbl->idx);
+    
+    
   }  
-
+  
   list_free(tables);
-
+  
   return ntables;
 }
 /*-------------------------------------------------------------------------*/
@@ -221,7 +238,7 @@ int load_hashes_pwdump(ophcrack_t *crack, int id) {
   
   list_nd_t *nd;
   
-  for (nd = hashes->head; nd != 0; nd = nd->prev) {
+  for (nd = hashes->head; nd != 0; nd = nd->next) {
     hash_t *hsh = (hash_t*)nd->data;
     ophcrack_add_hash(crack, hsh);
   }
@@ -287,7 +304,7 @@ int load_hashes_sam(ophcrack_t *crack, int id) {
   
   list_nd_t *nd;
   
-  for (nd = hashes->head; nd != 0; nd = nd->prev) {
+  for (nd = hashes->head; nd != 0; nd = nd->next) {
     hash_t *hsh = (hash_t*)nd->data;
     ophcrack_add_hash(crack, hsh);
   }
@@ -392,7 +409,7 @@ void display_load(ophcrack_t *crack, msg_load_t *msg, char *info) {
   table_t *tbl = msg->tbl;
   int which = msg->which;
   int done  = msg->done;
-  int size = msg->size >> 20;
+  uint64_t size = msg->size >> 20;
 
   char *name = tbl->name;
   int tidx   = tbl->idx;
@@ -411,7 +428,7 @@ void display_load(ophcrack_t *crack, msg_load_t *msg, char *info) {
   else if (which == 3)
     fprintf(arg->lfile, ".start file ");
 
-  fprintf(arg->lfile, "(%d MB) of table %s,%d\n", size, name, tidx);
+  fprintf(arg->lfile, "(%llu MB) of table %s,%d\n", (long long unsigned)size, name, tidx);
 }
 /*-------------------------------------------------------------------------*/
 void display_found(ophcrack_t *crack, msg_found_t *found, char *info) {
@@ -435,32 +452,62 @@ void display_found(ophcrack_t *crack, msg_found_t *found, char *info) {
      an empty pwd. */
 
   fprintf(arg->lfile, "%s", info);
-  
-  if (strcmp(pwd, "") == 0 && showempty)
-    fprintf(arg->lfile, "Found empty password for ");
-  else
-    fprintf(arg->lfile, "Found password %s for ", pwd);
 
-  switch (hsh->kind) {
-  case lm1:
-    fprintf(arg->lfile, "1st LM hash #%d", hid);
-    break;
+  if (!arg->auditmode) {
 
-  case lm2:
-    fprintf(arg->lfile, "2nd LM hash #%d", hid);
-    break;
-
-  case nt:
-    if (strcmp(hinfo, "") == 0)
-      fprintf(arg->lfile, "NT hash #%d", hid);
+    if (strcmp(pwd, "") == 0 && showempty)
+      fprintf(arg->lfile, "Found empty password for ");
     else
-      fprintf(arg->lfile, "user %s (NT hash #%d)", hinfo, hid);
-    break;
+      fprintf(arg->lfile, "Found password %s for ", pwd);
+    
+    switch (hsh->kind) {
+    case lm1:
+      fprintf(arg->lfile, "1st LM hash #%d", hid);
+      break;
+      
+    case lm2:
+      fprintf(arg->lfile, "2nd LM hash #%d", hid);
+      break;
+      
+    case nt:
+      if (strcmp(hinfo, "") == 0)
+	fprintf(arg->lfile, "NT hash #%d", hid);
+      else 
+	if (arg->hideuname)
+	  fprintf(arg->lfile, "user ***** (NT hash #%d)", hid);
+	else
+	  fprintf(arg->lfile, "user %s (NT hash #%d)", hinfo, hid);
+      break;
+      
+    default:
+      break;
+    }
+  } else {
 
-  default:
-    break;
+    if (strcmp(pwd, "") == 0 && showempty)
+      fprintf(arg->lfile, "Found empty password for ");
+    else
+      fprintf(arg->lfile, "Found password of length %d for ", hsh->length);
+    
+    switch (hsh->kind) {
+    case lm1:
+      fprintf(arg->lfile, "1st LM hash #%d", hid);
+      break;
+      
+    case lm2:
+      fprintf(arg->lfile, "2nd LM hash #%d", hid);
+      break;
+      
+    case nt:
+      fprintf(arg->lfile, "NT hash #%d", hid);
+      break;
+      
+    default:
+      break;
+    }
+    fprintf(arg->lfile, " category: %s ", category_string(hsh->category));
+    
   }
-
   if (tbl != 0)
     fprintf(arg->lfile, "in table %s #%d at column %d.", tbl->name, tbl->idx, col);
 
@@ -521,7 +568,7 @@ void display_status(ophcrack_t *crack, char *info) {
     int total_cmax = 0;
     int count = 0;
 
-    for (nd = tables->head; nd != 0; nd = nd->prev) {
+    for (nd = tables->head; nd != 0; nd = nd->next) {
       table_t *tbl = (table_t*)nd->data;
 
       if (!tbl->enabled) continue;
@@ -561,7 +608,7 @@ void handle_messages(ophcrack_t *crack) {
   struct timeval now;
   gettimeofday(&now, 0);
 
-  long tv_sec = now.tv_sec - tm_start.tv_sec;
+  long tv_sec = now.tv_sec - tm_main_start.tv_sec;
   long hour   = tv_sec / 3600;
   long sec    = tv_sec - 3600 * hour;
   long min    = sec / 60;
@@ -608,10 +655,14 @@ void handle_messages(ophcrack_t *crack) {
   
   if (pwdfound) {
     FILE *ofile = fopen(crack->arg->ofname, "w");
+    int fields[] = {1,1,1,1,1,1,1,1,1,1,1,1,1};
     
     if (ofile) {
       pthread_mutex_lock(mutex);
-      ophcrack_save(crack, ofile, 0, 0);
+      if (crack->arg->exportcsv) 
+	ophcrack_export_csv(crack, ofile, fields, ';', 0);
+      else
+	ophcrack_save(crack, ofile, 0, 0);
       pthread_mutex_unlock(mutex);
       
       fclose(ofile);
@@ -632,7 +683,7 @@ void handle_messages(ophcrack_t *crack) {
 }
 /*-------------------------------------------------------------------------*/
 int main(int argc, char **argv) {
-  static const char options[] = "bBc:Dd:ef:ghl:n:o:qrsS:t:uvw:";
+  static const char options[] = "aAbBc:Dd:ef:ghiIl:n:o:p:qrsS:t:uvw:x";
 
   arg_t *arg = arg_alloc();
   int c;
@@ -648,15 +699,31 @@ int main(int argc, char **argv) {
 
   int i = 1;
 
-  while (i < argc && strcmp(argv[i++], "-c") != 0);
+  while (i < argc && strcmp(argv[i++], "-c") != 0) {};
   if (i < argc) arg->cfname = strdup(argv[i]);
 
   load_config(arg);
 
   // Parse the command line.
 
+#ifdef OSX
+  // On Mac OS X, double-clicked applications have only one argument
+  // of the form "-psn...". We therefore do not parse the arguments if
+  // that one is present.
+
+  if (argc >= 2 && strncmp(argv[1], "-psn", 4) != 0) {
+#endif
+
   while ((c = getopt(argc, argv, options)) != -1)
     switch (c) {
+    case 'a':
+      arg->auditmode = 0;
+      break;
+
+    case 'A':
+      arg->auditmode = 1;
+      break;
+
     case 'b':
       arg->bforce = 0;
       break;
@@ -689,6 +756,14 @@ int main(int argc, char **argv) {
     case 'h':
       usage();
       break;
+
+    case 'i':
+      arg->hideuname = 1;
+      break;
+      
+    case 'I':
+      arg->hideuname = 0;
+      break;
       
     case 'l':
       arg->lfname = strdup(optarg);
@@ -702,6 +777,10 @@ int main(int argc, char **argv) {
       arg->ofname = strdup(optarg);
       break;
 
+    case 'p':
+      arg->preload = atoi(optarg);
+      break;
+
     case 'q':
       arg->quiet = 1;
       break;
@@ -735,10 +814,17 @@ int main(int argc, char **argv) {
       arg->samdir = strdup(optarg);
       break;
 
+    case 'x':
+      arg->exportcsv = 1;
+
     default:
       break;
     }
 
+#ifdef OSX
+  }
+#endif
+
   // If a log file has been provided, then we open it.
 
   if (arg->lfname != 0) {
@@ -859,7 +945,7 @@ int main(int argc, char **argv) {
 
   list_clear(enabled);
 
-  for (nd = tables->head; nd != 0; nd = nd->prev) {
+  for (nd = tables->head; nd != 0; nd = nd->next) {
     table_t *tbl = (table_t*)nd->data;
 
     if (tbl->enabled)
@@ -871,10 +957,10 @@ int main(int argc, char **argv) {
   list_t *hashes = crack->hashes;
   list_nd_t *tnd, *hnd;
 
-  for (hnd = hashes->head; hnd != 0; hnd = hnd->prev) {
+  for (hnd = hashes->head; hnd != 0; hnd = hnd->next) {
     hash_t *hsh = (hash_t*)hnd->data;
 
-    for (tnd = enabled->head; tnd != 0; tnd = tnd->prev) {
+    for (tnd = enabled->head; tnd != 0; tnd = tnd->next) {
       table_t *tbl = (table_t*)tnd->data;
       ophcrack_associate(crack, hsh, tbl);
     }
@@ -886,7 +972,8 @@ int main(int argc, char **argv) {
   int count = 0;
   fsm = fsm_alloc(crack);
 
-  gettimeofday(&tm_start, 0);
+  gettimeofday(&tm_main_start, 0);
+  tm_main_total.tv_sec = 0;
   fsm_handle_start(fsm);
 
   while (fsm->state != st_wait) {
