@@ -26,8 +26,8 @@
  *   that compiling, linking, and/or using OpenSSL is allowed.
  *   
  *   
- *   $Rev: 118 $
- *   $Date: 2008-04-29 13:49:51 +0200 (Tue, 29 Apr 2008) $
+ *   $Rev: 147 $
+ *   $Date: 2009-05-07 15:25:33 +0200 (Thu, 07 May 2009) $
  *   
  *   
 */
@@ -37,6 +37,7 @@
 #include <QMenu>
 #include <assert.h>
 #include <math.h>
+#include <config.h>
 
 #include "ophcrackgui.h"
 #include "hash.h"
@@ -57,6 +58,10 @@ OphcrackGUI::OphcrackGUI(ophcrack_t *crack) : QMainWindow() {
   samDialog = 0;
   tableDialog = 0;
   helpDialog = 0;
+  exportDialog = 0;
+#if HAVE_QWT
+  graphDialog = 0;
+#endif
 
   fsm_reset_preload(fsm);
   fsm_reset_bforce(fsm);
@@ -64,7 +69,7 @@ OphcrackGUI::OphcrackGUI(ophcrack_t *crack) : QMainWindow() {
 
   // Setup the model which will display the hashes.
 
-  hashModel = new HashModel(hashView);
+  hashModel = new HashModel(hashView, arg->auditmode);
   hashView->setModel(hashModel);
   hashSelModel = hashView->selectionModel();
 
@@ -83,7 +88,7 @@ OphcrackGUI::OphcrackGUI(ophcrack_t *crack) : QMainWindow() {
 
   list_nd_t *nd;
 
-  for (nd = crack->hashes->head; nd != 0; nd = nd->prev) {
+  for (nd = crack->hashes->head; nd != 0; nd = nd->next) {
     hash_t *hsh = (hash_t*)nd->data;
     hashModel->insertHash(hsh);
   }
@@ -103,7 +108,7 @@ OphcrackGUI::OphcrackGUI(ophcrack_t *crack) : QMainWindow() {
 
   list_t *table_str = crack->arg->table_str_cmd;
   
-  for (nd = table_str->head; nd != 0; nd = nd->prev) {
+  for (nd = table_str->head; nd != 0; nd = nd->next) {
     char *tblstr = (char*)nd->data;
     installTables(tblstr, true, false);
   }
@@ -115,7 +120,7 @@ OphcrackGUI::OphcrackGUI(ophcrack_t *crack) : QMainWindow() {
 
   table_str = crack->arg->table_str_conf;
 
-  for (nd = table_str->head; nd != 0; nd = nd->prev) {
+  for (nd = table_str->head; nd != 0; nd = nd->next) {
     char *tblstr = (char*)nd->data;
     installTables(tblstr, replace, false);
   }
@@ -129,6 +134,10 @@ OphcrackGUI::OphcrackGUI(ophcrack_t *crack) : QMainWindow() {
   tableModel->insertRoot(lmgermanv2);
   tableModel->insertRoot(ntextended);
   tableModel->insertRoot(ntdict);
+  tableModel->insertRoot(ntnine);
+  tableModel->insertRoot(nteight);
+  tableModel->insertRoot(ntnum);
+  tableModel->insertRoot(ntseven);
 
   // Associate a menu to the load button.
 
@@ -153,18 +162,35 @@ OphcrackGUI::OphcrackGUI(ophcrack_t *crack) : QMainWindow() {
 
   loadButton->setMenu(loadMenu);
 
+  // Associate a menu to the save button.
+
+  saveMenu   = new QMenu();
+  saveFile   = saveMenu->addAction("Save to file");
+  exportFile = saveMenu->addAction("Export to CSV");
+
+  connect(saveFile, SIGNAL(triggered()), this, SLOT(save()));
+  connect(exportFile, SIGNAL(triggered()), this, SLOT(exportCSV()));
+
+  saveButton->setMenu(saveMenu);
+
   // Connect the other buttons.
 
   connect(deleteButton, SIGNAL(clicked()), this, SLOT(delSelection()));
-  connect(saveButton, SIGNAL(clicked()), this, SLOT(save()));
   connect(crackButton, SIGNAL(clicked()), this, SLOT(startStopCrack()));
   connect(tableButton, SIGNAL(clicked()), this, SLOT(selectTables()));
   connect(resetButton, SIGNAL(clicked()), this, SLOT(resetStat()));
+#if HAVE_QWT
+  connect(graphButton, SIGNAL(clicked()), this, SLOT(displayGraph()));
+#else
+  graphButton->setEnabled(false);
+#endif
   connect(threadSlider, SIGNAL(valueChanged(int)), this, SLOT(threadChanged(int)));
   connect(hreduxSlider, SIGNAL(valueChanged(int)), this, SLOT(hreduxChanged(int)));
   connect(queueSlider, SIGNAL(valueChanged(int)), this, SLOT(queueChanged(int)));
   connect(defaultButton, SIGNAL(clicked()), this, SLOT(defaultConfig()));
   connect(bforceBox, SIGNAL(currentIndexChanged(int)), this, SLOT(bforceChanged(int)));
+  connect(hideUnameBox, SIGNAL(currentIndexChanged(int)), this, SLOT(hideUnameChanged(int)));
+  connect(auditModeBox, SIGNAL(currentIndexChanged(int)), this, SLOT(auditModeChanged(int)));
   connect(aboutButton, SIGNAL(clicked()), this, SLOT(displayAbout()));
   connect(helpButton, SIGNAL(clicked()), this, SLOT(displayHelp()));
   connect(sessBox, SIGNAL(stateChanged(int)), this, SLOT(sessOnOff(int)));
@@ -291,7 +317,7 @@ void OphcrackGUI::tablesChanged(void) {
 
   htbl_t **id_to_htbl = (htbl_t**)malloc(maxtid*sizeof(htbl_t*)); 
 
-  for (hnd = hashes->head; hnd != 0; hnd = hnd->prev) {
+  for (hnd = hashes->head; hnd != 0; hnd = hnd->next) {
     hash_t *hsh = (hash_t*)hnd->data;
 
     // We store the tables associated with the current hash into the
@@ -316,7 +342,7 @@ void OphcrackGUI::tablesChanged(void) {
 
     list_t *enabled = crack->enabled;
 
-    for (tnd = enabled->head; tnd != 0; tnd = tnd->prev) {
+    for (tnd = enabled->head; tnd != 0; tnd = tnd->next) {
       table_t *tbl = (table_t*)tnd->data;
       htbl_t *htbl = id_to_htbl[tbl->id];
 
@@ -359,12 +385,12 @@ void OphcrackGUI::hashesAdded(list_t *hashes) {
   list_t *enabled = crack->enabled;
   list_nd_t *hnd, *tnd;
 
-  for (hnd = hashes->head; hnd != 0; hnd = hnd->prev) {
+  for (hnd = hashes->head; hnd != 0; hnd = hnd->next) {
     hash_t *hsh = (hash_t*)hnd->data;
 
     assert(hsh->tables->size == 0);
 
-    for (tnd = enabled->head; tnd != 0; tnd = tnd->prev) {
+    for (tnd = enabled->head; tnd != 0; tnd = tnd->next) {
       table_t *tbl = (table_t*)tnd->data;
       ophcrack_associate(crack, hsh, tbl);
     }
@@ -437,7 +463,7 @@ void OphcrackGUI::loadSingleHash(void) {
     // potentially crack.
 
     else {
-      for (nd = hashes->head; nd != 0; nd = nd->prev) {
+      for (nd = hashes->head; nd != 0; nd = nd->next) {
 	hash_t *hsh = (hash_t*)nd->data;
 
 	ophcrack_add_hash(crack, hsh);
@@ -483,7 +509,7 @@ void OphcrackGUI::loadHashFile(void) {
       int maxhid = crack->maxhid;
       hash_load_pwdump(hashes, file, maxhid);
 
-      for (nd = hashes->head; nd != 0; nd = nd->prev) {
+      for (nd = hashes->head; nd != 0; nd = nd->next) {
 	hash_t *hsh = (hash_t*)nd->data;
 	
 	ophcrack_add_hash(crack, hsh);
@@ -547,7 +573,7 @@ void OphcrackGUI::loadSamFile(void) {
     
     // Add the hashes.
     
-    for (nd = hashes->head; nd != 0; nd = nd->prev) {
+    for (nd = hashes->head; nd != 0; nd = nd->next) {
       hash_t *hsh = (hash_t*)nd->data;
       
       ophcrack_add_hash(crack, hsh);
@@ -572,7 +598,7 @@ void OphcrackGUI::loadLocalSam(void) {
   // list of hashes.
 
   if (ret > 0) {
-    for (nd = hashes->head; nd != 0; nd = nd->prev) {
+    for (nd = hashes->head; nd != 0; nd = nd->next) {
       hash_t *hsh = (hash_t*)nd->data;
       
       ophcrack_add_hash(crack, hsh);
@@ -617,7 +643,7 @@ void OphcrackGUI::loadRemoteSam(void) {
       // list of hashes.
       
       if (ret > 0) {
-	for (nd = hashes->head; nd != 0; nd = nd->prev) {
+	for (nd = hashes->head; nd != 0; nd = nd->next) {
 	  hash_t *hsh = (hash_t*)nd->data;
 	  
 	  ophcrack_add_hash(crack, hsh);
@@ -710,6 +736,52 @@ void OphcrackGUI::save(void) {
   }
 }
 //---------------------------------------------------------------------------
+void OphcrackGUI::exportCSV(void) {
+  QString fileName;
+  int *fields;
+
+  fields = (int*)malloc(13*sizeof(int));
+  if (exportDialog == 0)
+    exportDialog = new ExportDialog(this);
+  
+  if (exportDialog->exec() == QDialog::Accepted) {
+    exportDialog->getFields(fields);
+    fileName = QFileDialog::getSaveFileName(this, 
+					    tr("Export to CSV"),
+					    QDir::currentPath(), 
+					    QString());
+
+    if (!fileName.isNull()) {
+      const char *fname = fileName.toLatin1().constData();
+      
+      FILE *file = fopen(fname, "w");
+      
+      if (file == 0) {
+	QMessageBox msgBox(QMessageBox::Warning,
+			   "Warning",
+			 QString("Cannot open file %1 for writing.").arg(fname),
+			   QMessageBox::Ok);
+	msgBox.exec();
+      } 
+      
+      else {
+	scheduler_t *sched = crack->sched;
+	pthread_mutex_t *mutex = sched->mutex;
+	
+	pthread_mutex_lock(mutex);
+	ophcrack_export_csv(crack, file, fields, 
+			    exportDialog->getSeparator(), 
+			    exportDialog->getQuotes());
+	pthread_mutex_unlock(mutex);
+	
+	fclose(file);
+      }
+    }
+    
+  }
+  free(fields);
+}
+//---------------------------------------------------------------------------
 void OphcrackGUI::resetStat(void) {
   scheduler_t *sched = crack->sched;
   pthread_mutex_t *mutex = sched->mutex;
@@ -723,6 +795,19 @@ void OphcrackGUI::resetStat(void) {
   updateStatus();
 }
 //---------------------------------------------------------------------------
+void OphcrackGUI::displayGraph(void) {
+#if HAVE_QWT
+  if (graphDialog == 0) {
+    graphDialog = new GraphDialog(this);
+  }
+
+  int tv_sec = (int) tm_total.tv_sec;
+  
+  graphDialog->plot(crack->stat, tv_sec);
+  graphDialog->show();
+#endif
+}
+//---------------------------------------------------------------------------
 void OphcrackGUI::handleMessages(void) {
   message_t *msg = 0;
 
@@ -739,7 +824,13 @@ void OphcrackGUI::handleMessages(void) {
     if (msg->kind == msg_found) {
       msg_found_t *found = (msg_found_t*)msg->data;
       hash_t *hsh = found->hsh;
-      
+
+#if HAVE_QWT
+      int tv_sec = (int) tm_total.tv_sec;
+
+      if (graphDialog != 0 && graphDialog->isVisible()) 
+	graphDialog->plot(crack->stat, tv_sec);
+#endif
       hashModel->updateHash(hsh);
     }
 
@@ -815,6 +906,8 @@ void OphcrackGUI::startCrack(void) {
 
   gettimeofday(&tm_start, 0);
   tm_sess = tm_start;
+  extern struct timeval tm_main_start;
+  tm_main_start = tm_start;
 
   showNotFound(false);
 }
@@ -893,6 +986,11 @@ void OphcrackGUI::delSelection(void) {
   crack->maxhid = 0;
   crack->npwds_total = 0;
   crack->npwds_found = 0;
+  for (int i=0; i<16; i++) {
+    crack->stat->length[i]=0;
+    crack->stat->category[i]=0;
+  }
+  list_clear(crack->stat->time);
 
   // Remove all the lines in the hash view. Since the number of rows
   // changes, we always remove the first one.
@@ -998,6 +1096,7 @@ void OphcrackGUI::updateStat(void) {
 
   startField->clear();
   startField->insert(QString("%1").arg(start));
+
 }
 //---------------------------------------------------------------------------
 void OphcrackGUI::updateProgress(void) {
@@ -1069,9 +1168,11 @@ void OphcrackGUI::updateStatus(void) {
 
   if (fsm->state & (~st_wait & ~st_pause1 & ~st_pause2)) {
     struct timeval tm_stop;
+    extern struct timeval tm_main_total;
     gettimeofday(&tm_stop, 0);
   
     tm_total.tv_sec += tm_stop.tv_sec - tm_start.tv_sec;
+    tm_main_total.tv_sec = tm_total.tv_sec;
     tm_start = tm_stop;
   }
 
@@ -1104,10 +1205,12 @@ void OphcrackGUI::defaultConfig(void) {
 }
 //---------------------------------------------------------------------------
 void OphcrackGUI::updateConfig(void) {
-  int nthreads = arg->nthreads;
-  int nhredux  = arg->nhredux;
-  int mdqueue  = arg->mdqueue;
-  int bforce   = arg->bforce;
+  int nthreads  = arg->nthreads;
+  int nhredux   = arg->nhredux;
+  int mdqueue   = arg->mdqueue;
+  int bforce    = arg->bforce;
+  int hideuname = arg->hideuname;
+  int auditmode = arg->auditmode;
 
   threadSlider->setSliderPosition(nthreads);
   threadChanged(nthreads);
@@ -1120,6 +1223,10 @@ void OphcrackGUI::updateConfig(void) {
 
   bforceBox->setCurrentIndex(bforce > 0 ? 1 : 0);
 
+  hideUnameBox->setCurrentIndex(hideuname > 0 ? 1 : 0);
+
+  auditModeBox->setCurrentIndex(auditmode > 0 ? 1 : 0);
+
   if (arg->sfname != 0) {
     sessEdit->setText(arg->sfname);
     sessBox->setCheckState(arg->ssave ? Qt::Checked : Qt::Unchecked);
@@ -1153,6 +1260,36 @@ void OphcrackGUI::bforceChanged(int x) {
   updateStatus();
 }
 //---------------------------------------------------------------------------
+void OphcrackGUI::hideUnameChanged(int x) {
+  arg->hideuname = x;
+
+  hashModel->setUnameVisible(x);
+
+  list_nd_t *nd;
+  for (nd = crack->hashes->head; nd != 0; nd = nd->next) {
+    hash_t *hsh = (hash_t*)nd->data;
+    hashModel->updateHash(hsh);
+  }
+
+}
+//---------------------------------------------------------------------------
+void OphcrackGUI::auditModeChanged(int x) {
+  arg->auditmode = x;
+
+  hashModel->setAuditMode(x);
+
+  list_nd_t *nd;
+  for (nd = crack->hashes->head; nd != 0; nd = nd->next) {
+    hash_t *hsh = (hash_t*)nd->data;
+    hashModel->updateHash(hsh);
+  }
+  
+  if (x)
+    hideUnameBox->setEnabled(false);
+  else
+    hideUnameBox->setEnabled(true);
+}
+//---------------------------------------------------------------------------
 void OphcrackGUI::saveConfig(void) {
   FILE *file = fopen(arg->cfname, "w");
 
@@ -1164,7 +1301,7 @@ void OphcrackGUI::saveConfig(void) {
   QMap<QString, int> paths;
   list_nd_t *nd;
 
-  for (nd = arg->table_path->head; nd != 0; nd = nd->prev) {
+  for (nd = arg->table_path->head; nd != 0; nd = nd->next) {
     QString path = QString((char*)nd->data);
     
     if (path[0] != 0)
@@ -1219,6 +1356,8 @@ void OphcrackGUI::saveConfig(void) {
   fprintf(file, "nhredux %d\n", arg->nhredux);
   fprintf(file, "maxdqueue %d\n", arg->mdqueue);
   fprintf(file, "bforce %d\n", arg->bforce);
+  fprintf(file, "hideuname %d\n", arg->hideuname);
+  fprintf(file, "auditmode %d\n", arg->auditmode);
 
   if (arg->sfname != 0) {
     fprintf(file, "session_file %s\n", arg->sfname);
@@ -1266,7 +1405,7 @@ void OphcrackGUI::installTables(QString str, bool replace, bool warn) {
   list_t *tables = list_alloc();
   list_nd_t *dnd;
 
-  for (dnd = table_path->head; dnd != 0; dnd = dnd->prev) {
+  for (dnd = table_path->head; dnd != 0; dnd = dnd->next) {
     char *dir = (char*)dnd->data;
     int ret = table_open(tables, dir, tblstr);
 
@@ -1293,11 +1432,29 @@ void OphcrackGUI::installTables(QString str, bool replace, bool warn) {
     table_t *tbl = (table_t*)tables->head->data;
     table_kind_t kind = tbl->kind;
 
+    // Check the size of the tables. 
+
+    int size_invalid = 0;
+    list_nd_t *nd;
+
+    for (nd = tables->head; nd != 0; nd = nd->next) {
+      table_t *tbl = (table_t*)nd->data;
+      ophcrack_setup_table(tbl);
+      if (table_verify(tbl) < 0) {
+	size_invalid = 1;
+	QMessageBox msgBox(QMessageBox::Warning,
+			   "Warning",
+			   QString("Size of table %1 is invalid.").arg(str) +
+			   QString("Please download it again."),
+			   QMessageBox::Ok);
+	msgBox.exec();
+      }
+    }
+
     // Check if we must replace existing tables by the newly loaded
     // ones.
 
     bool installed = tableModel->isInstalled(kind);
-    list_nd_t *nd;
 
     if (replace && installed) {
       int ntables = crack->tables->size;
@@ -1313,20 +1470,21 @@ void OphcrackGUI::installTables(QString str, bool replace, bool warn) {
       tableModel->uninstall(kind);
  
       // Install the newly loaded tables.
-      
-      for (nd = tables->head; nd != 0; nd = nd->prev) {
-	table_t *tbl = (table_t*)nd->data;
-	ophcrack_add_table(crack, tbl);
+      if (!size_invalid) {
+	for (nd = tables->head; nd != 0; nd = nd->next) {
+	  table_t *tbl = (table_t*)nd->data;
+	  ophcrack_add_table(crack, tbl);
+	}
+	
+	tableModel->install(tables);
       }
-      
-      tableModel->install(tables);
     }
 
     // If the tables are not installed yet, then we install them. If
     // 'replace' is false then we disable all tables.
 
-    else if (!installed) {
-      for (nd = tables->head; nd != 0; nd = nd->prev) {
+    else if (!installed && !size_invalid) {
+      for (nd = tables->head; nd != 0; nd = nd->next) {
 	table_t *tbl = (table_t*)nd->data;
 
 	if (!replace) tbl->enabled = 0;
@@ -1393,7 +1551,7 @@ void OphcrackGUI::showNotFound(bool flag) {
   list_t *hashes = crack->hashes;
   list_nd_t *nd;
 
-  for (nd = hashes->head; nd != 0; nd = nd->prev) {
+  for (nd = hashes->head; nd != 0; nd = nd->next) {
     hash_t *hsh = (hash_t*)nd->data;
 
     if (hsh->done <= 0 && hsh->id >= 0)
