@@ -26,8 +26,8 @@
  *   that compiling, linking, and/or using OpenSSL is allowed.
  *   
  *   
- *   $Rev: 124 $
- *   $Date: 2008-05-05 14:11:52 +0200 (Mon, 05 May 2008) $
+ *   $Rev: 148 $
+ *   $Date: 2009-05-08 13:37:36 +0200 (Fri, 08 May 2009) $
  *   
  *   
 */
@@ -49,6 +49,10 @@
 #include "lmgerman.h"
 #include "ntextended.h"
 #include "ntdict.h"
+#include "ntnine.h"
+#include "nteight.h"
+#include "ntnum.h"
+#include "ntseven.h"
 #include "message.h"
 /*-------------------------------------------------------------------------*/
 ophcrack_t *ophcrack_alloc(int nthreads, arg_t *arg) {
@@ -89,16 +93,12 @@ void ophcrack_stop(ophcrack_t *crack) {
   scheduler_wait(crack->sched);
 }
 /*-------------------------------------------------------------------------*/
-int ophcrack_add_table(ophcrack_t *crack, table_t *tbl) {
-  scheduler_t *sched = crack->sched;
-  pthread_mutex_t *mutex = sched->mutex;
+int ophcrack_setup_table(table_t *tbl) {
 
-  /* Setup the table. We set the kind and id variable ourselves and
+  /* Setup the table. We set the kind variable ourselves and
      then call the appropriate setup function. */
   
   tbl->kind = table_kind(tbl->code);
-  tbl->id   = crack->maxtid++;
-
   int ret = 0;
 
   switch (tbl->kind) {
@@ -130,20 +130,41 @@ int ophcrack_add_table(ophcrack_t *crack, table_t *tbl) {
     ret = ntdict_setup(tbl);
     break;
 
+  case ntnine:
+    ret = ntnine_setup(tbl);
+    break;
+
+  case nteight:
+    ret = nteight_setup(tbl);
+    break;
+
+  case ntnum:
+    ret = ntnum_setup(tbl);
+    break;
+
+  case ntseven:
+    ret = ntseven_setup(tbl);
+    break;
+
   case unknown:
+    ret = -1;
     break;
   }
 
-  if (ret < 1) return ret;
+  return ret;
+}
+/*-------------------------------------------------------------------------*/
+void ophcrack_add_table(ophcrack_t *crack, table_t *tbl) {
+  scheduler_t *sched = crack->sched;
+  pthread_mutex_t *mutex = sched->mutex;
 
+  tbl->id   = crack->maxtid++;
   tbl->cmin = tbl->ncols;
   tbl->cmax = tbl->ncols;
 
   pthread_mutex_lock(mutex);
   list_add_tail(crack->tables, tbl);
   pthread_mutex_unlock(mutex);
-
-  return 1;
 }
 /*-------------------------------------------------------------------------*/
 void ophcrack_add_hash(ophcrack_t *crack, hash_t *hsh) {
@@ -510,9 +531,9 @@ ophtask_t *ophcrack_find(ophcrack_t *crack, ophtask_t *task, int id) {
       uint64_t postfix = tosearch[i]->postfix;
 
       fprintf(arg->lfile, 
-	      "prefix = %u, postfix = %llu.\n", 
+	      "prefix = %u, postfix = %u.\n", 
 	      (unsigned int)prefix, 
-	      (long long unsigned int)postfix);
+	      (unsigned int)postfix);
 
       pthread_mutex_unlock(mutex);
     }
@@ -591,7 +612,7 @@ ophtask_t *ophcrack_lookup_idx(ophcrack_t *crack, ophtask_t *task, int id) {
       fprintf(arg->lfile, "#%d in table %s,%d ", hid, name, idx);
       fprintf(arg->lfile, "at column %d, ", cmax-i);
       fprintf(arg->lfile, "with prefix = %u ", prefix);
-      fprintf(arg->lfile, "and postfix = %llu ... ", (long long unsigned int)postfix);
+      fprintf(arg->lfile, "and postfix = %u ... ", (unsigned int)postfix);
     }
 
     tbl->lookup_idx(hsh, tbl, tosearch[i]);
@@ -673,7 +694,7 @@ ophtask_t *ophcrack_lookup_end(ophcrack_t *crack, ophtask_t *task, int id) {
       fprintf(arg->lfile, "#%d in table %s,%d ", hid, name, idx);
       fprintf(arg->lfile, "at column %d, ", cmax-i);
       fprintf(arg->lfile, "with prefix = %u, ", prefix);
-      fprintf(arg->lfile, "postfix = %llu, ", (long long unsigned int)postfix);
+      fprintf(arg->lfile, "postfix = %u, ", (unsigned int)postfix);
       fprintf(arg->lfile, "low = %u and high = %u ... ", low, high);
     }
 
@@ -693,7 +714,7 @@ ophtask_t *ophcrack_lookup_end(ophcrack_t *crack, ophtask_t *task, int id) {
       if (ret == 0)
 	fprintf(arg->lfile, "no offset found.\n");
       else
-	fprintf(arg->lfile, "offset = %u.\n", tosearch[i]->offset);
+	fprintf(arg->lfile, "offset = %llu.\n", (long long unsigned int)tosearch[i]->offset);
 
       pthread_mutex_unlock(mutex);
     }
@@ -773,14 +794,14 @@ ophtask_t *ophcrack_lookup_srt(ophcrack_t *crack, ophtask_t *task, int id) {
       uint64_t postfix = tosearch[i]->postfix;
       uint32_t low  = tosearch[i]->low;
       uint32_t high = tosearch[i]->high;
-      uint32_t offset = tosearch[i]->offset;
+      uint64_t offset = tosearch[i]->offset;
       
       fprintf(arg->lfile, "#%d in table %s,%d ", hid, name, idx);
       fprintf(arg->lfile, "at column %d, ", cmax-i);
       fprintf(arg->lfile, "with prefix = %u, ", prefix);
-      fprintf(arg->lfile, "postfix = %llu, ", (long long unsigned int)postfix);
+      fprintf(arg->lfile, "postfix = %u, ", (unsigned int)postfix);
       fprintf(arg->lfile, "low = %u, high = %u ", low, high);
-      fprintf(arg->lfile, "and offset = %u ... ", offset);
+      fprintf(arg->lfile, "and offset = %llu ... ", (long long unsigned int)offset);
     }
     
     tbl->lookup_srt(hsh, tbl, tosearch[i]);
@@ -845,14 +866,14 @@ ophtask_t *ophcrack_check(ophcrack_t *crack, ophtask_t *task, int id) {
       uint64_t postfix = tosearch[i]->postfix;
       uint32_t low  = tosearch[i]->low;
       uint32_t high = tosearch[i]->high;
-      uint32_t offset = tosearch[i]->offset;
+      uint64_t offset = tosearch[i]->offset;
       
       fprintf(arg->lfile, "#%d in table %s,%d ", hid, name, idx);
       fprintf(arg->lfile, "at column %d, ", cmax-i);
       fprintf(arg->lfile, "with prefix = %u, ", prefix);
-      fprintf(arg->lfile, "postfix = %llu, ", (long long unsigned int)postfix);
+      fprintf(arg->lfile, "postfix = %u, ", (unsigned int)postfix);
       fprintf(arg->lfile, "low = %u, high = %u, ", low, high);
-      fprintf(arg->lfile, "offset = %u ", offset);
+      fprintf(arg->lfile, "offset = %llu ", (long long unsigned int)offset);
       fprintf(arg->lfile, "and start = ");
       fprintf_hex(arg->lfile, tosearch[i]->pwd, 16);
       fprintf(arg->lfile, " ... ");
@@ -948,7 +969,7 @@ ophtask_t *ophcrack_bforce_all(ophcrack_t *crack, ophtask_t *task, int id) {
     force->nhashes = nhashes;
     force->hashes  = (hash_t**)malloc(nhashes*sizeof(hash_t*));
 
-    for (nd=hashes_lm->head, n=0; nd!=0; nd=nd->prev, ++n)
+    for (nd=hashes_lm->head, n=0; nd!=0; nd=nd->next, ++n)
       force->hashes[n] = nd->data;
 
     for (i=1; i<=maxlen; ++i)
@@ -972,7 +993,7 @@ ophtask_t *ophcrack_bforce_all(ophcrack_t *crack, ophtask_t *task, int id) {
     force->nhashes = nhashes;
     force->hashes  = (hash_t**)malloc(nhashes*sizeof(hash_t*));
 
-    for (nd=hashes_nt->head, n=0; nd!=0; nd=nd->prev, ++n)
+    for (nd=hashes_nt->head, n=0; nd!=0; nd=nd->next, ++n)
       force->hashes[n] = nd->data;
 
     for (i=1; i<=maxlen; ++i)
@@ -1042,7 +1063,9 @@ ophtask_t *ophcrack_bforce_nt(ophcrack_t *crack, ophtask_t *task, int id) {
 
   hash_t **hashes = force->hashes;
   int nhashes = force->nhashes;
+  int *bloom = force->bloom;
   int n;
+  int bloom_byte, bloom_bit;
 
   if (force->hash == 0) {
     assert(force->found == 0);
@@ -1055,6 +1078,11 @@ ophtask_t *ophcrack_bforce_nt(ophcrack_t *crack, ophtask_t *task, int id) {
 
       force->hash[n] = (uchar_t*)malloc(16*sizeof(uchar_t));
       memcpy(force->hash[n], hsh->hash, 16);
+
+      bloom_byte = (* (int*)(force->hash[n])) & 0xfffff;
+      bloom_bit = 1 << (bloom_byte >> 15);
+      bloom[bloom_byte & 0x7fff] |= bloom_bit;
+      
     }
   }
 
@@ -1094,17 +1122,22 @@ ophtask_t *ophcrack_bforce_nt(ophcrack_t *crack, ophtask_t *task, int id) {
       
       ntextended_mkhash(pwd, hash);
 
-      for (n=0; n<nhashes; ++n) {
-	int found = force->found[n];
+      bloom_byte = (* (int*)hash) & 0xfffff;
+      bloom_bit = 1 << (bloom_byte >> 15);
 
-	if (!found && memcmp(force->hash[n], hash, 16) == 0) {
-	  force->found[n] = 1;
+      if (bloom[(bloom_byte & 0x7fff)] & bloom_bit) {
 
-	  /* Do not report deleted hashes as found. */
-
-	  if (hashes[n]->id >= 0) {
-	    ophcrack_found(crack, hashes[n], 0, 0, (char*)pwd);
-	    ++task->stat->match_bforce;
+	for (n=0; n<nhashes; ++n) {
+	  int found = force->found[n];
+	  
+	  if (!found && memcmp(force->hash[n], hash, 16) == 0) {
+	    force->found[n] = 1;
+	    /* Do not report deleted hashes as found. */
+	    
+	    if (hashes[n]->id >= 0) {
+	      ophcrack_found(crack, hashes[n], 0, 0, (char*)pwd);
+	      ++task->stat->match_bforce;
+	    }
 	  }
 	}
       }
@@ -1167,7 +1200,9 @@ ophtask_t *ophcrack_bforce_lm(ophcrack_t *crack, ophtask_t *task, int id) {
 
   hash_t **hashes = force->hashes;
   int nhashes = force->nhashes;
+  int *bloom = force->bloom;
   int n;
+  int bloom_byte, bloom_bit;
 
   if (force->hash == 0) {
     assert(force->found == 0);
@@ -1180,6 +1215,11 @@ ophtask_t *ophcrack_bforce_lm(ophcrack_t *crack, ophtask_t *task, int id) {
 
       force->hash[n] = (uchar_t*)malloc(8*sizeof(uchar_t));
       memcpy(force->hash[n], hsh->hash, 8);
+
+      bloom_byte = (* (int*)(force->hash[n])) & 0xfffff;
+      bloom_bit = 1 << (bloom_byte >> 15);
+      bloom[bloom_byte & 0x7fff] |= bloom_bit;
+      
     }
   }
 
@@ -1219,17 +1259,22 @@ ophtask_t *ophcrack_bforce_lm(ophcrack_t *crack, ophtask_t *task, int id) {
 
       lmtable_mkhash(pwd, hash);
 
-      for (n=0; n<nhashes; ++n) {
-	int found = force->found[n];
+      bloom_byte = (* (int*)hash) & 0xfffff;
+      bloom_bit = 1 << (bloom_byte >> 15);
 
-	if (!found && memcmp(force->hash[n], hash, 8) == 0) {
-	  force->found[n] = 1;
+      if (bloom[(bloom_byte & 0x7fff)] & bloom_bit) {
 
-	  /* Do not report deleted hashes as found. */
-
-	  if (hashes[n]->id >= 0) {
-	    ophcrack_found(crack, hashes[n], 0, 0, (char*)pwd);
-	    ++task->stat->match_bforce;
+	for (n=0; n<nhashes; ++n) {
+	  int found = force->found[n];
+	  
+	  if (!found && memcmp(force->hash[n], hash, 8) == 0) {
+	    force->found[n] = 1;
+	    /* Do not report deleted hashes as found. */
+	    
+	    if (hashes[n]->id >= 0) {
+	      ophcrack_found(crack, hashes[n], 0, 0, (char*)pwd);
+	      ++task->stat->match_bforce;
+	    }
 	  }
 	}
       }
@@ -1290,6 +1335,10 @@ int ophcrack_find_cmin(int ncols, int cmax, int total) {
 
   int cmin = ncols-(int)x;
 
+  /* Make sure that we progress, even if arg->nhredux < chain length */
+  if (cmin >= cmax)
+    cmin = cmax-1;
+
   return cmin < 0 ? 0 : cmin;
 }
 /*-------------------------------------------------------------------------*/
@@ -1297,7 +1346,7 @@ int ophcrack_has_next(ophcrack_t *crack) {
   list_t *hashes = crack->hashes;
   list_nd_t *hnd;
 
-  for (hnd = hashes->head; hnd != 0; hnd = hnd->prev) {
+  for (hnd = hashes->head; hnd != 0; hnd = hnd->next) {
     hash_t *hsh = hnd->data;
 
     list_t *tables = hsh->tables;
@@ -1305,7 +1354,7 @@ int ophcrack_has_next(ophcrack_t *crack) {
 
     if (hsh->done != 0) continue;
 
-    for (tnd = tables->head; tnd != 0; tnd = tnd->prev) {
+    for (tnd = tables->head; tnd != 0; tnd = tnd->next) {
       htbl_t *htbl = tnd->data;
       table_t *tbl = htbl->tbl;
 
@@ -1344,7 +1393,7 @@ int ophcrack_next(ophcrack_t *crack) {
       /* Search for a valid table. */
 
       while ((!tbl->active || htbl->col < 0) && --ntables) {
-	tnd  = tnd->prev == 0 ? tables->head : tnd->prev;
+	tnd  = tnd->next == 0 ? tables->head : tnd->next;
 	htbl = tnd->data;
 	tbl  = htbl->tbl;
       }
@@ -1376,18 +1425,18 @@ int ophcrack_next(ophcrack_t *crack) {
 	   next time and move to the next table. */
 
 	htbl->col = cmin-1;
-	hsh->tnd  = tnd->prev;
+	hsh->tnd  = tnd->next;
       }
 
       /* Otherwise, we go to the next hash. */
 
       else {
 	assert(ntables == 0);
-	hsh->tnd = tnd->prev;       /* Probably not needed. */
+	hsh->tnd = tnd->next;       /* Probably not needed. */
       }
     }
 
-    crack->hnd = hnd->prev;
+    crack->hnd = hnd->next;
   }
 
   pthread_mutex_unlock(mutex);
@@ -1449,6 +1498,9 @@ void ophcrack_found(ophcrack_t *crack, hash_t *hsh, table_t *tbl, int col,
 
   char *lmpwd1 = 0;
   char *lmpwd2 = 0;
+
+  extern struct timeval tm_main_total;
+  extern struct timeval tm_main_start;
   
   /* We must check the 'done' flag because somebody may have already
      cracked the hash before we reach this point. Furthermore, it
@@ -1463,24 +1515,58 @@ void ophcrack_found(ophcrack_t *crack, hash_t *hsh, table_t *tbl, int col,
     if (hsh->pwd != pwd) strcpy(hsh->pwd, pwd);
     hsh->done = 1;
 
+    if (hsh->length == 0) hsh->length = strlen(pwd);
+    hsh->table = (char*) malloc(32*sizeof(char));
+    if (tbl == 0) 
+      snprintf(hsh->table, 32, "Bruteforce");
+    else
+      snprintf(hsh->table, 32, "%s", table_string(tbl->kind));
+    hsh->category = categorize_password(hsh->pwd);
+    
+    if (hsh->time == -1) {
+      if (tm_main_total.tv_sec == 0) {
+	if (tm_main_start.tv_sec == 0)
+	  hsh->time = 0;
+	else {
+	  struct timeval now;
+	  gettimeofday(&now, 0);
+	  long tv_sec = now.tv_sec - tm_main_start.tv_sec;
+	  hsh->time = (int) tv_sec;
+	}
+      } else 
+	hsh->time = (int) tm_main_total.tv_sec;
+    }
+
     message_found(hsh, tbl, col);
     
     /* A NT hash has been cracked. */
 
-    if (hsh->kind == nt)
+    if (hsh->kind == nt) {
       ++crack->npwds_found;
+      ++crack->stat->length[(hsh->length > 15) ? 15 : hsh->length];
+      ++crack->stat->category[hsh->category];
+      list_add_tail(crack->stat->time, &(hsh->time));
+    }
 
     /* A 1st LM hash which is not associated to a NT hash and whose
        corresponding 2nd LM hash has been cracked. */
 
-    else if (hsh->kind == lm1 && hsh->lmhsh1 == 0 && hsh->lmhsh2->done > 0)
+    else if (hsh->kind == lm1 && hsh->lmhsh1 == 0 && hsh->lmhsh2->done > 0) {
       ++crack->npwds_found;
+      ++crack->stat->length[(hsh->length > 15) ? 15 : hsh->length];
+      ++crack->stat->category[hsh->category];
+      list_add_tail(crack->stat->time, &(hsh->time));
+    }
 
     /* A 2nd LM hash which is not associated to a NT hash and whose
        corresponding 1st LM hash has been cracked. */
 
-    else if (hsh->kind == lm2 && hsh->lmhsh2 == 0 && hsh->lmhsh1->done > 0)
+    else if (hsh->kind == lm2 && hsh->lmhsh2 == 0 && hsh->lmhsh1->done > 0) {
       ++crack->npwds_found;
+      ++crack->stat->length[(hsh->length > 15) ? 15 : hsh->length];
+      ++crack->stat->category[hsh->category];
+      list_add_tail(crack->stat->time, &(hsh->time));
+    }
 
     /* If both LM hashes have been cracked, then we schedule a brute
        force task to try to crack the corresponding NT hash. */
@@ -1616,6 +1702,7 @@ void ophcrack_preload_check(ophcrack_t *crack) {
 ophtask_t *ophcrack_preload_all(ophcrack_t *crack, ophtask_t *task, int id) {
   scheduler_t *sched = crack->sched;
   ophload_t *load = task->data;
+  int force_preload = crack->arg->preload;
 
   list_t *tables = load->tables;
   list_t *active = crack->active;
@@ -1636,7 +1723,7 @@ ophtask_t *ophcrack_preload_all(ophcrack_t *crack, ophtask_t *task, int id) {
   /* Mark all the tables as inactive and unload all the tables which
      have been preloaded but are now disabled. */
 
-  for (nd = crack->tables->head; nd != 0; nd = nd->prev) {
+  for (nd = crack->tables->head; nd != 0; nd = nd->next) {
     table_t *tbl = nd->data;
 
     tbl->active = 0;  
@@ -1667,14 +1754,14 @@ ophtask_t *ophcrack_preload_all(ophcrack_t *crack, ophtask_t *task, int id) {
      preloaded active tables to the list of inactive tables. */
 
   if (tables->size == 0) {
-    for (nd = active->head; nd != 0; nd = nd->prev) { 
+    for (nd = active->head; nd != 0; nd = nd->next) { 
       table_t *tbl = nd->data;
 
       if (preload[tbl->id] > 0)
 	list_add_tail(inactive, tbl);
     }
 
-    for (nd = remaining->head; nd != 0; nd = nd->prev) {
+    for (nd = remaining->head; nd != 0; nd = nd->next) {
       table_t *tbl = nd->data;
       list_add_tail(tables, tbl);
     }
@@ -1709,9 +1796,24 @@ ophtask_t *ophcrack_preload_all(ophcrack_t *crack, ophtask_t *task, int id) {
   list_clear(active);
   list_clear(remaining);
 
+
+  /* If we force the preload, we unload all the tables in use and restart
+     from scratch. */
+
+  if (force_preload < 3) 
+    while (tables->size > 0) {
+      table_t *tbl = list_rem_head(tables);
+      if (preload[tbl->id] > 0) freeram += tbl->idxsize;
+      if (preload[tbl->id] > 1) freeram += tbl->endsize;
+      if (preload[tbl->id] > 2) freeram += tbl->srtsize;
+      
+      preload[tbl->id] = 0;
+      list_add_tail(remaining, tbl);
+    }
+
   while (tables->size > 0) {
     table_t *tbl = list_rem_head(tables);
-    ulong_t size = 0;
+    uint64_t size = 0;
 
     assert(tbl->enabled);
 
@@ -1738,7 +1840,7 @@ ophtask_t *ophcrack_preload_all(ophcrack_t *crack, ophtask_t *task, int id) {
 
       while (size > freeram && inactive->size > 0) {
 	table_t *tbl = inactive->head->data;
-	ulong_t size = 0;
+	uint64_t size = 0;
 
 	assert(preload[tbl->id] > 0);
 
@@ -1765,7 +1867,7 @@ ophtask_t *ophcrack_preload_all(ophcrack_t *crack, ophtask_t *task, int id) {
 	   after another. */
 
 	if (preload[tbl->id] > 0) {
-	  ulong_t size = 0;
+	  uint64_t size = 0;
 
 	  if (preload[tbl->id] == 3) size += tbl->srtsize;
 	  else if (preload[tbl->id] == 2) size += tbl->endsize;
@@ -1791,7 +1893,7 @@ ophtask_t *ophcrack_preload_all(ophcrack_t *crack, ophtask_t *task, int id) {
       /* At this point, either there is enough space or all the
 	 previously preloaded and active tables have been unloaded. */
       
-      if (size <= freeram) {
+      if (size <= freeram ) {
 	freeram -= size;
 	preload[tbl->id] = 3;
 	list_add_tail(active, tbl);
@@ -1823,7 +1925,7 @@ ophtask_t *ophcrack_preload_all(ophcrack_t *crack, ophtask_t *task, int id) {
 
   for (i=0; i<n; ++i) {
     table_t *tbl = list_rem_head(remaining);
-    ulong_t size = 0;
+    uint64_t size = 0;
 
     assert(preload[tbl->id] == 0);
 
@@ -1833,7 +1935,7 @@ ophtask_t *ophcrack_preload_all(ophcrack_t *crack, ophtask_t *task, int id) {
 
     /* Check if we can preload all the files. */
 
-    if (size <= freeram) {
+    if (size <= freeram && force_preload == 3) {
       freeram -= size;
       
       preload[tbl->id] = 3;
@@ -1847,7 +1949,7 @@ ophtask_t *ophcrack_preload_all(ophcrack_t *crack, ophtask_t *task, int id) {
     else if (flag) {
       size -= tbl->srtsize;
       
-      if (size <= freeram) {
+      if (size <= freeram && force_preload > 1) {
 	freeram -= size;
 	
 	preload[tbl->id] = 2;
@@ -1855,7 +1957,10 @@ ophtask_t *ophcrack_preload_all(ophcrack_t *crack, ophtask_t *task, int id) {
       } else
 	list_add_tail(remaining, tbl);
 
-      flag = 0;
+      /* If we want to force the preload of less files, we do not change
+	 the flag in order to load more than one table if possible. */
+
+      if (force_preload == 3) flag = 0;
     }
 
     /* If the table cannot be preloaded either completely or
@@ -1867,47 +1972,82 @@ ophtask_t *ophcrack_preload_all(ophcrack_t *crack, ophtask_t *task, int id) {
 
   /* If no tables could be preloaded, then we try to preload at least
      the .index file of the first remaining table. */
+  
+  if (active->size == 0 && remaining->size > 0 ) {
+    if (force_preload > 1) {
+      table_t *tbl = remaining->head->data;
+      uint64_t size = 0;
+      
+      assert(preload[tbl->id] == 0);
+      size += tbl->idxsize;
+      
+      if (size <= freeram && force_preload > 0) {
+	freeram -= size;
+	
+	preload[tbl->id] = 1;
+	list_rem_head(remaining);
+	list_add_tail(active, tbl);
+      }
+    } else {
+      
+      /* If we force to preload only the index files, then we want to
+	 preload all the indexes we can. */
+      
+      n = remaining->size;
+      
+      for (i=0; i<n; ++i) {
+	table_t *tbl = list_rem_head(remaining);
+	uint64_t size = 0;
+	
+	assert(preload[tbl->id] == 0);
 
-  if (active->size == 0 && remaining->size > 0) {
-    table_t *tbl = remaining->head->data;
-    ulong_t size = 0;
-    
-    assert(preload[tbl->id] == 0);
-    size += tbl->idxsize;
-
-    if (size <= freeram) {
-      freeram -= size;
-
-      preload[tbl->id] = 1;
-      list_rem_head(remaining);
-      list_add_tail(active, tbl);
+	size += tbl->idxsize;
+	
+	if (size <= freeram && force_preload > 0) {
+	  freeram -= size;
+	  
+	  preload[tbl->id] = 1;
+	  list_add_tail(active, tbl);
+	} else
+	  list_add_tail(remaining, tbl);
+      }
     }
   }
-
+  
   /* We mark all the newly preloaded tables as active. */
   
-  for (nd = active->head; nd != 0; nd = nd->prev) {
+  for (nd = active->head; nd != 0; nd = nd->next) {
     table_t *tbl = nd->data;
     tbl->active = 1;
   }
 
   /* We mark all the remaining tables as inactive if at least one
      table has been preloaded. We mark only the first one as active
-     otherwise. */
+     otherwise. 
 
-  for (nd = remaining->head; nd != 0; nd = nd->prev) {
-    table_t *tbl = nd->data;
-    tbl->active = 0;
-  }
+     If we force not to preload, then we mark all tables as active. */
+  
+  if (force_preload > 0) {
+    for (nd = remaining->head; nd != 0; nd = nd->next) {
+      table_t *tbl = nd->data;
+      tbl->active = 0;
+    }
    
-  if (active->size == 0 && remaining->size > 0) {
-    table_t *tbl = list_rem_head(remaining);
-
-    tbl->active = 1;
-    list_add_tail(active, tbl);
-
-    assert(active->size == 1);
-  } 
+    if (active->size == 0 && remaining->size > 0) {
+      table_t *tbl = list_rem_head(remaining);
+      
+      tbl->active = 1;
+      list_add_tail(active, tbl);
+      
+      assert(active->size == 1);
+    } 
+  } else {
+    while (remaining->size > 0) {
+      table_t *tbl = list_rem_head(remaining);
+      tbl->active = 1;
+      list_add_tail(active, tbl);
+    }
+  }
 
   list_free(inactive);
   assert(active->size + remaining->size == ntables);
@@ -1921,7 +2061,7 @@ ophtask_t *ophcrack_preload_all(ophcrack_t *crack, ophtask_t *task, int id) {
     /* Schedule an unload task for all the tables for which the desired
        status is lower than the current one. */
     
-    for (nd = enabled->head; nd != 0; nd = nd->prev) {
+    for (nd = enabled->head; nd != 0; nd = nd->next) {
       table_t *tbl = nd->data;
       int n = tbl->preload - preload[tbl->id];
       
@@ -1946,7 +2086,7 @@ ophtask_t *ophcrack_preload_all(ophcrack_t *crack, ophtask_t *task, int id) {
     /* Schedule a preload task for all the active tables for which the
        desired preload status is higher than the current one. */
     
-    for (nd = active->head; nd != 0; nd = nd->prev) {
+    for (nd = active->head; nd != 0; nd = nd->next) {
       table_t *tbl = nd->data;
       
       if (preload[tbl->id] > tbl->preload) {
@@ -2072,12 +2212,12 @@ void ophcrack_update(ophcrack_t *crack) {
 
   pthread_mutex_lock(mutex);
 
-  for (hnd = hashes->head; hnd != 0; hnd = hnd->prev) {
+  for (hnd = hashes->head; hnd != 0; hnd = hnd->next) {
     hash_t *hsh = hnd->data;
     list_t *tables = hsh->tables;
     list_nd_t *tnd;
 
-    for (tnd = tables->head; tnd != 0; tnd = tnd->prev) {
+    for (tnd = tables->head; tnd != 0; tnd = tnd->next) {
       htbl_t *htbl = tnd->data;
       table_t *tbl = htbl->tbl;
 
@@ -2115,7 +2255,7 @@ void ophcrack_update(ophcrack_t *crack) {
   list_t *active = crack->active;
   list_nd_t *nd;
 
-  for (nd = active->head; nd != 0; nd = nd->prev) {
+  for (nd = active->head; nd != 0; nd = nd->next) {
     table_t *tbl = nd->data;
 
     if (tbl->active && tbl->cmax == 0) {
@@ -2139,8 +2279,10 @@ void ophcrack_save(ophcrack_t *crack, FILE *file, int nice, int status) {
   
   list_t *hashes = crack->hashes;
   list_nd_t *nd;
+
+  arg_t *arg = crack->arg;
   
-  for (nd = hashes->head; nd != 0; nd = nd->prev) {
+  for (nd = hashes->head; nd != 0; nd = nd->next) {
     hash_t *hsh = nd->data;
     id_to_hash[hsh->id] = hsh;
   }
@@ -2151,7 +2293,225 @@ void ophcrack_save(ophcrack_t *crack, FILE *file, int nice, int status) {
   
   for (i=0; i<maxhid; ++i) {
     assert(id_to_hash[i] != 0);
-    hash_print(id_to_hash[i], file, nice, status);
+    hash_print(id_to_hash[i], file, nice, status, arg->hideuname);
+  }
+
+  free(id_to_hash);
+}
+/*-------------------------------------------------------------------------*/
+void ophcrack_export_csv(ophcrack_t *crack, FILE *file, int *fields, char separator, char quote) {
+
+  /* We need to be careful to write each line only once. */
+
+  int maxhid = crack->maxhid;
+  hash_t **id_to_hash = (hash_t**)calloc(maxhid, sizeof(hash_t*));
+  
+  list_t *hashes = crack->hashes;
+  list_nd_t *nd;
+
+  
+  for (nd = hashes->head; nd != 0; nd = nd->next) {
+    hash_t *hsh = nd->data;
+    id_to_hash[hsh->id] = hsh;
+  }
+  
+  /* Store the hashes in the order they have been read. */
+  
+  int i,first;
+
+  for (i=0; i<maxhid; ++i) {
+    assert(id_to_hash[i] != 0);
+
+    first = 1;
+    hash_t *hsh = id_to_hash[i];
+    hash_kind_t kind = hsh->kind;
+  
+    /* Find the associated LM and NT hashes. */
+    
+    hash_t *lmhsh1 = 0;
+    hash_t *lmhsh2 = 0;
+    hash_t *nthsh  = 0;
+    int type       = 0;
+    
+    switch (kind) {
+    case lm1:
+      lmhsh1 = hsh;
+      lmhsh2 = hsh->lmhsh2;
+      nthsh  = hsh->lmhsh1;
+      if (lmhsh2 == 0 && nthsh == 0) type = 1;
+      if (lmhsh2 != 0 && nthsh == 0) type = 2;
+      break;
+      
+    case lm2:
+      lmhsh1 = hsh->lmhsh1;
+      lmhsh2 = hsh;
+      nthsh  = hsh->lmhsh2;
+      if (lmhsh1 != 0 && nthsh == 0) type = 2;
+      break;
+      
+    case nt:
+      lmhsh1 = hsh->lmhsh1;
+      lmhsh2 = hsh->lmhsh2;
+      nthsh  = hsh;
+      if (lmhsh1 != 0 && lmhsh2 != 0) type = 3;
+      if (lmhsh1 == 0 && lmhsh2 == 0) type = 4;
+      break;
+      
+    default:
+      break;
+    }
+
+    if (type) {
+
+      if (fields[0]) {
+	if (!first) fprintf(file,"%c", separator);
+	if (quote) fprintf(file, "%c%d%c", quote, hsh->id, quote);
+	else fprintf(file, "%d", hsh->id); 
+	first = 0;
+      }
+      
+      if (fields[1]) {
+	if (!first) fprintf(file,"%c", separator);
+	if (quote) {if (hsh->uid) fprintf(file, "%c%d%c", quote, hsh->uid, quote); }
+	else {if (hsh->uid) fprintf(file, "%d", hsh->uid); }
+	first = 0;
+      }
+      
+      if (fields[2]) {
+	if (!first) fprintf(file,"%c", separator);
+	if (quote) {if (hsh->info[0]) fprintf(file, "%c%s%c", quote, hsh->info, quote); }
+	else {if (hsh->info[0]) fprintf(file, "%s", hsh->info); }
+	first = 0;
+      }
+      
+      if (fields[3]) {
+	if (!first) fprintf(file,"%c", separator);
+	if (quote) {if (lmhsh1) fprintf(file, "%c%s%c", quote, lmhsh1->str, quote); }
+	else {if (lmhsh1) fprintf(file, "%s", lmhsh1->str); }
+	first = 0;
+      }
+      
+      if (fields[4]) {
+	if (!first) fprintf(file,"%c", separator);
+	if (quote) {if (nthsh) fprintf(file, "%c%s%c", quote, nthsh->str, quote); }
+	else {if (nthsh) fprintf(file, "%s", nthsh->str);}
+	first = 0;
+      }
+      
+      if (fields[5]) {
+	if (!first) fprintf(file,"%c", separator);
+	if (quote) {if (lmhsh1) fprintf(file, "%c%s%c", quote, lmhsh1->pwd, quote);} 
+	else {if (lmhsh1) fprintf(file, "%s", lmhsh1->pwd);}
+	first = 0;
+      }
+      
+      if (fields[6]) {
+	if (!first) fprintf(file,"%c", separator);
+	if (quote) {if (lmhsh2) fprintf(file, "%c%s%c", quote, lmhsh2->pwd, quote);}
+	else {if (lmhsh2) fprintf(file, "%s", lmhsh2->pwd); }
+	first = 0;
+      }
+      
+      if (fields[7]) {
+	if (!first) fprintf(file,"%c", separator);
+	if (quote) {if (nthsh) fprintf(file, "%c%s%c", quote, nthsh->pwd, quote); }
+	else {if (nthsh) fprintf(file, "%s", nthsh->pwd);}
+	first = 0;
+      }
+      
+      if (fields[8]) {
+	if (!first) fprintf(file,"%c", separator);
+	switch (type) {
+	case 1: if (quote) fprintf(file, "%c%d%c", quote, lmhsh1->done, quote);
+	  else fprintf(file, "%d", lmhsh1->done);
+	  break;
+	case 2: if (quote) fprintf(file, "%c%d%c", quote, lmhsh1->done && lmhsh2->done, quote);
+	  else fprintf(file, "%d", lmhsh1->done && lmhsh2->done);
+	  break;
+	case 3:
+	case 4: 
+	default: if (quote) fprintf(file, "%c%d%c", quote, nthsh->done, quote);
+	  else fprintf(file, "%d", nthsh->done);
+	  break;
+	}
+	first = 0;
+      }
+      
+      if (fields[9]) {
+	if (!first) fprintf(file,"%c", separator);
+	switch (type) {
+	case 1: if (quote) {if (lmhsh1->done > 0) fprintf(file, "%c%d%c", quote, lmhsh1->time, quote);}
+	  else {if (lmhsh1->done > 0) fprintf(file, "%d", lmhsh1->time);}
+	  break;
+	case 2: if (quote) {if (lmhsh1->done > 0 && lmhsh2->done > 0) fprintf(file, "%c%d%c", quote, (lmhsh1->time > lmhsh2->time) ? lmhsh1->time : lmhsh2->time, quote);}
+	  else {if (lmhsh1->done > 0 && lmhsh2->done > 0) fprintf(file, "%d", (lmhsh1->time > lmhsh2->time) ? lmhsh1->time : lmhsh2->time);}
+	  break;
+	case 3:
+	case 4: 
+	default: if (quote) {if (nthsh->done > 0) fprintf(file, "%c%d%c", quote, nthsh->time, quote);}
+	  else {if (nthsh->done > 0) fprintf(file, "%d", nthsh->time);}
+	  break;
+	}
+	first = 0;
+      }
+      
+      if (fields[10]) {
+	if (!first) fprintf(file,"%c", separator);
+	switch (type) {
+	case 1: if (quote) {if (lmhsh1->done > 0) fprintf(file, "%c%d%c", quote, lmhsh1->length, quote);}
+	  else {if (lmhsh1->done > 0) fprintf(file, "%d", lmhsh1->length);}
+	  break;
+	case 2: if (quote) {if (lmhsh1->done > 0 && lmhsh2->done > 0) fprintf(file, "%c%d%c", quote, lmhsh1->length + lmhsh2->length, quote);}
+	  else {if (lmhsh1->done > 0 && lmhsh2->done > 0) fprintf(file, "%d", lmhsh1->length + lmhsh2->length);}
+	  break;
+	case 3:
+	case 4: 
+	default: if (quote) {if (nthsh->done) fprintf(file, "%c%d%c", quote, nthsh->length, quote);}
+	  else {if (nthsh->done) fprintf(file, "%d", nthsh->length);}
+	  break;
+	}
+	first = 0;
+      }
+      
+      if (fields[11]) {
+	if (!first) fprintf(file,"%c", separator);
+	switch (type) {
+	case 1: if (quote) {if (lmhsh1->done > 0) fprintf(file, "%c%s%c", quote, category_string(lmhsh1->category), quote);}
+	  else {if (lmhsh1->done > 0) fprintf(file, "%s", category_string(lmhsh1->category));}
+	  break;
+	case 2: if (quote) {if (lmhsh1->done > 0 && lmhsh2->done > 0) fprintf(file, "%c%s%c", quote, category_string(lmhsh1->category|lmhsh2->category), quote);}
+	  else {if (lmhsh1->done > 0 && lmhsh2->done > 0) fprintf(file, "%s", category_string(lmhsh1->category|lmhsh2->category));}
+	  break;
+	case 3:
+	case 4: 
+	default: if (quote) {if (nthsh->done) fprintf(file, "%c%s%c", quote, category_string(nthsh->category), quote);}
+	  else {if (nthsh->done) fprintf(file, "%s", category_string(nthsh->category));}
+	  break;
+	}
+	first = 0;
+      }
+      
+      if (fields[12]) {
+	if (!first) fprintf(file,"%c", separator);
+	switch (type) {
+	case 1: if (quote) {if (lmhsh1->done > 0) fprintf(file, "%c%s%c", quote, lmhsh1->table, quote);}
+	  else {if (lmhsh1->done > 0) fprintf(file, "%s", lmhsh1->table);}
+	  break;
+	case 2: if (quote) {if (lmhsh1->done > 0 && lmhsh2->done > 0) fprintf(file, "%c%s/%s%c", quote, lmhsh1->table, lmhsh2->table, quote);}
+	  else {if (lmhsh1->done > 0 && lmhsh2->done > 0) fprintf(file, "%s/%s", lmhsh1->table, lmhsh2->table);}
+	  break;
+	case 3: if (quote) {if (nthsh->done) fprintf(file, "%c%s/%s%c", quote, lmhsh1->table, lmhsh2->table, quote);}
+	  else {if (nthsh->done) fprintf(file, "%s/%s", lmhsh1->table, lmhsh2->table);}
+	  break;
+	case 4: 
+	default: if (quote) {if (nthsh->done) fprintf(file, "%c%s%c", quote, nthsh->table, quote);}
+	  else {if (nthsh->done) fprintf(file, "%s", nthsh->table);}
+	  break;
+	}
+	first = 0;
+      }
+      fprintf(file, "\n");
+    }
   }
 
   free(id_to_hash);
